@@ -43,17 +43,16 @@
 #include <libmissioncontrol/mc-account.h>
 
 #include <libempathy/empathy-utils.h>
-#include <libempathy/empathy-debug.h>
-#include <libempathy/empathy-conf.h>
 
 #include "empathy-chat-view.h"
 #include "empathy-chat.h"
-#include "empathy-preferences.h"
+#include "empathy-conf.h"
 #include "empathy-theme-manager.h"
 #include "empathy-ui-utils.h"
 #include "empathy-smiley-manager.h"
 
-#define DEBUG_DOMAIN "ChatView"
+#define DEBUG_FLAG EMPATHY_DEBUG_CHAT
+#include <libempathy/empathy-debug.h>
 
 /* Number of seconds between timestamps when using normal mode, 5 minutes. */
 #define TIMESTAMP_INTERVAL 300
@@ -62,20 +61,17 @@
 #define MAX_SCROLL_TIME 0.4 /* seconds */
 #define SCROLL_DELAY 33     /* milliseconds */
 
-#define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), EMPATHY_TYPE_CHAT_VIEW, EmpathyChatViewPriv))
-
-struct _EmpathyChatViewPriv {
+#define GET_PRIV(obj) EMPATHY_GET_PRIV (obj, EmpathyChatView)
+typedef struct {
 	GtkTextBuffer *buffer;
 
 	EmpathyTheme   *theme;
 
 	time_t         last_timestamp;
-	EmpathyChatViewBlock last_block_type;
 
 	gboolean       allow_scrolling;
 	guint          scroll_timeout;
 	GTimer        *scroll_time;
-	gboolean       is_group_chat;
 
 	GtkTextMark   *find_mark_previous;
 	GtkTextMark   *find_mark_next;
@@ -89,10 +85,8 @@ struct _EmpathyChatViewPriv {
 
 	guint          notify_system_fonts_id;
 	guint          notify_show_avatars_id;
-};
+} EmpathyChatViewPriv;
 
-static void     empathy_chat_view_class_init          (EmpathyChatViewClass      *klass);
-static void     empathy_chat_view_init                (EmpathyChatView           *view);
 static void     chat_view_finalize                   (GObject                  *object);
 static gboolean chat_view_drag_motion                (GtkWidget                *widget,
 						      GdkDragContext           *context,
@@ -151,20 +145,15 @@ empathy_chat_view_class_init (EmpathyChatViewClass *klass)
 static void
 empathy_chat_view_init (EmpathyChatView *view)
 {
-	EmpathyChatViewPriv *priv;
-	gboolean            show_avatars;
-
-	priv = GET_PRIV (view);
+	gboolean             show_avatars;
+	EmpathyChatViewPriv *priv = G_TYPE_INSTANCE_GET_PRIVATE (view,
+		EMPATHY_TYPE_CHAT_VIEW, EmpathyChatViewPriv);
 
+	view->priv = priv;
 	priv->buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
-
-	priv->last_block_type = EMPATHY_CHAT_VIEW_BLOCK_NONE;
 	priv->last_timestamp = 0;
-
 	priv->allow_scrolling = TRUE;
 
-	priv->is_group_chat = FALSE;
-
 	g_object_set (view,
 		      "wrap-mode", GTK_WRAP_WORD_CHAR,
 		      "editable", FALSE,
@@ -216,7 +205,7 @@ chat_view_finalize (GObject *object)
 	view = EMPATHY_CHAT_VIEW (object);
 	priv = GET_PRIV (view);
 
-	empathy_debug (DEBUG_DOMAIN, "finalize: %p", object);
+	DEBUG ("finalize: %p", object);
 
 	empathy_conf_notify_remove (empathy_conf_get (), priv->notify_system_fonts_id);
 	empathy_conf_notify_remove (empathy_conf_get (), priv->notify_show_avatars_id);
@@ -634,20 +623,10 @@ chat_view_theme_changed_cb (EmpathyThemeManager *manager,
 {
 	EmpathyChatViewPriv *priv;
 	gboolean            show_avatars = FALSE;
-	gboolean            theme_rooms = FALSE;
 
 	priv = GET_PRIV (view);
 
-	priv->last_block_type = EMPATHY_CHAT_VIEW_BLOCK_NONE;
-
-	empathy_conf_get_bool (empathy_conf_get (),
-			      EMPATHY_PREFS_CHAT_THEME_CHAT_ROOM,
-			      &theme_rooms);
-	if (!theme_rooms && priv->is_group_chat) {
-		empathy_theme_manager_apply (manager, view, NULL);
-	} else {
-		empathy_theme_manager_apply_saved (manager, view);
-	}
+	empathy_theme_manager_apply_saved (manager, view);
 
 	/* Needed for now to update the "rise" property of the names to get it
 	 * vertically centered.
@@ -715,31 +694,19 @@ chat_view_avatar_cache_data_free (gpointer ptr)
 GdkPixbuf *
 empathy_chat_view_get_avatar_pixbuf_with_cache (EmpathyContact *contact)
 {
-	static GHashTable *avatar_cache = NULL;
 	AvatarData        *data;
 	EmpathyAvatar     *avatar;
 	GdkPixbuf         *tmp_pixbuf;
 	GdkPixbuf         *pixbuf = NULL;
 
-	/* Init avatar cache */
-	if (!avatar_cache) {
-		avatar_cache = g_hash_table_new_full (empathy_contact_hash,
-						      empathy_contact_equal,
-						      g_object_unref,
-						      chat_view_avatar_cache_data_free);
-	}
-
 	/* Check if avatar is in cache and if it's up to date */
 	avatar = empathy_contact_get_avatar (contact);
-	data = g_hash_table_lookup (avatar_cache, contact);
+	data = g_object_get_data (G_OBJECT (contact), "chat-view-avatar-cache");
 	if (data) {
 		if (avatar && !tp_strdiff (avatar->token, data->token)) {
 			/* We have the avatar in cache */
 			return data->pixbuf;
 		}
-
-		/* The cache is outdate */
-		g_hash_table_remove (avatar_cache, contact);
 	}
 
 	/* Avatar not in cache, create pixbuf */
@@ -757,9 +724,8 @@ empathy_chat_view_get_avatar_pixbuf_with_cache (EmpathyContact *contact)
 	data->token = g_strdup (avatar->token);
 	data->pixbuf = pixbuf;
 
-	g_hash_table_insert (avatar_cache,
-			     g_object_ref (contact),
-			     data);
+	g_object_set_data_full (G_OBJECT (contact), "chat-view-avatar-cache",
+				data, chat_view_avatar_cache_data_free);
 
 	return data->pixbuf;
 }
@@ -770,14 +736,67 @@ empathy_chat_view_new (void)
 	return g_object_new (EMPATHY_TYPE_CHAT_VIEW, NULL);
 }
 
+/* Code stolen from pidgin/gtkimhtml.c */
+static gboolean
+chat_view_scroll_cb (EmpathyChatView *view)
+{
+	EmpathyChatViewPriv *priv;
+	GtkAdjustment      *adj;
+	gdouble             max_val;
+
+	priv = GET_PRIV (view);
+	adj = GTK_TEXT_VIEW (view)->vadjustment;
+	max_val = adj->upper - adj->page_size;
+
+	g_return_val_if_fail (priv->scroll_time != NULL, FALSE);
+
+	if (g_timer_elapsed (priv->scroll_time, NULL) > MAX_SCROLL_TIME) {
+		/* time's up. jump to the end and kill the timer */
+		gtk_adjustment_set_value (adj, max_val);
+		g_timer_destroy (priv->scroll_time);
+		priv->scroll_time = NULL;
+		priv->scroll_timeout = 0;
+		return FALSE;
+	}
+
+	/* scroll by 1/3rd the remaining distance */
+	gtk_adjustment_set_value (adj, gtk_adjustment_get_value (adj) + ((max_val - gtk_adjustment_get_value (adj)) / 3));
+	return TRUE;
+}
+
+void
+empathy_chat_view_scroll_down (EmpathyChatView *view)
+{
+	EmpathyChatViewPriv *priv;
+
+	g_return_if_fail (EMPATHY_IS_CHAT_VIEW (view));
+
+	priv = GET_PRIV (view);
+
+	if (!priv->allow_scrolling) {
+		return;
+	}
+
+	DEBUG ("Scrolling down");
+
+	if (priv->scroll_time) {
+		g_timer_reset (priv->scroll_time);
+	} else {
+		priv->scroll_time = g_timer_new();
+	}
+	if (!priv->scroll_timeout) {
+		priv->scroll_timeout = g_timeout_add (SCROLL_DELAY,
+						      (GSourceFunc) chat_view_scroll_cb,
+						      view);
+	}
+}
+
 void
 empathy_chat_view_append_message (EmpathyChatView *view,
 				  EmpathyMessage  *msg)
 {
 	EmpathyChatViewPriv *priv = GET_PRIV (view);
-	EmpathyContact      *sender;
 	gboolean             bottom;
-	gboolean             from_self;
 
 	g_return_if_fail (EMPATHY_IS_CHAT_VIEW (view));
 	g_return_if_fail (EMPATHY_IS_MESSAGE (msg));
@@ -787,8 +806,6 @@ empathy_chat_view_append_message (EmpathyChatView *view,
 	}
 
 	bottom = chat_view_is_scrolled_down (view);
-	sender = empathy_message_get_sender (msg);
-	from_self = empathy_contact_is_user (sender);
 	
 	chat_view_maybe_trim_buffer (view);
 
@@ -797,6 +814,11 @@ empathy_chat_view_append_message (EmpathyChatView *view,
 	if (bottom) {
 		empathy_chat_view_scroll_down (view);
 	}
+
+	if (priv->last_contact) {
+		g_object_unref (priv->last_contact);
+	}
+	priv->last_contact = g_object_ref (empathy_message_get_sender (msg));
 }
 
 void
@@ -821,7 +843,10 @@ empathy_chat_view_append_event (EmpathyChatView *view,
 		empathy_chat_view_scroll_down (view);
 	}
 
-	priv->last_block_type = EMPATHY_CHAT_VIEW_BLOCK_EVENT;
+	if (priv->last_contact) {
+		g_object_unref (priv->last_contact);
+		priv->last_contact = NULL;
+	}
 }
 
 void
@@ -891,77 +916,25 @@ empathy_chat_view_append_button (EmpathyChatView *view,
 		empathy_chat_view_scroll_down (view);
 	}
 
-	priv->last_block_type = EMPATHY_CHAT_VIEW_BLOCK_INVITE;
+	if (priv->last_contact) {
+		g_object_unref (priv->last_contact);
+		priv->last_contact = NULL;
+	}
 }
 
 void
 empathy_chat_view_scroll (EmpathyChatView *view,
 			 gboolean        allow_scrolling)
 {
-	EmpathyChatViewPriv *priv;
+	EmpathyChatViewPriv *priv = GET_PRIV (view);
 
 	g_return_if_fail (EMPATHY_IS_CHAT_VIEW (view));
 
-	priv = GET_PRIV (view);
+	DEBUG ("Scrolling %s", allow_scrolling ? "enabled" : "disabled");
 
 	priv->allow_scrolling = allow_scrolling;
-
-	empathy_debug (DEBUG_DOMAIN, "Scrolling %s",
-		      allow_scrolling ? "enabled" : "disabled");
-}
-
-/* Code stolen from pidgin/gtkimhtml.c */
-static gboolean
-chat_view_scroll_cb (EmpathyChatView *view)
-{
-	EmpathyChatViewPriv *priv;
-	GtkAdjustment      *adj;
-	gdouble             max_val;
-
-	priv = GET_PRIV (view);
-	adj = GTK_TEXT_VIEW (view)->vadjustment;
-	max_val = adj->upper - adj->page_size;
-
-	g_return_val_if_fail (priv->scroll_time != NULL, FALSE);
-
-	if (g_timer_elapsed (priv->scroll_time, NULL) > MAX_SCROLL_TIME) {
-		/* time's up. jump to the end and kill the timer */
-		gtk_adjustment_set_value (adj, max_val);
-		g_timer_destroy (priv->scroll_time);
-		priv->scroll_time = NULL;
-		priv->scroll_timeout = 0;
-		return FALSE;
-	}
-
-	/* scroll by 1/3rd the remaining distance */
-	gtk_adjustment_set_value (adj, gtk_adjustment_get_value (adj) + ((max_val - gtk_adjustment_get_value (adj)) / 3));
-	return TRUE;
-}
-
-void
-empathy_chat_view_scroll_down (EmpathyChatView *view)
-{
-	EmpathyChatViewPriv *priv;
-
-	g_return_if_fail (EMPATHY_IS_CHAT_VIEW (view));
-
-	priv = GET_PRIV (view);
-
-	if (!priv->allow_scrolling) {
-		return;
-	}
-
-	empathy_debug (DEBUG_DOMAIN, "Scrolling down");
-
-	if (priv->scroll_time) {
-		g_timer_reset (priv->scroll_time);
-	} else {
-		priv->scroll_time = g_timer_new();
-	}
-	if (!priv->scroll_timeout) {
-		priv->scroll_timeout = g_timeout_add (SCROLL_DELAY,
-						      (GSourceFunc) chat_view_scroll_cb,
-						      view);
+	if (allow_scrolling) {
+		empathy_chat_view_scroll_down (view);
 	}
 }
 
@@ -996,7 +969,6 @@ empathy_chat_view_clear (EmpathyChatView *view)
 	 */
 	priv = GET_PRIV (view);
 
-	priv->last_block_type = EMPATHY_CHAT_VIEW_BLOCK_NONE;
 	priv->last_timestamp = 0;
 }
 
@@ -1457,36 +1429,6 @@ empathy_chat_view_get_smiley_menu (GCallback    callback,
 	return menu;
 }
 
-/* FIXME: Do we really need this? Better to do it internally only at setup time,
- * we will never change it on the fly.
- */
-void
-empathy_chat_view_set_is_group_chat (EmpathyChatView *view,
-				    gboolean        is_group_chat)
-{
-	EmpathyChatViewPriv *priv;
-	gboolean            theme_rooms = FALSE;
-
-	g_return_if_fail (EMPATHY_IS_CHAT_VIEW (view));
-
-	priv = GET_PRIV (view);
-
-	priv->is_group_chat = is_group_chat;
-
-	empathy_conf_get_bool (empathy_conf_get (),
-			      EMPATHY_PREFS_CHAT_THEME_CHAT_ROOM,
-			      &theme_rooms);
-
-	if (!theme_rooms && is_group_chat) {
-		empathy_theme_manager_apply (empathy_theme_manager_get (),
-					    view,
-					    NULL);
-	} else {
-		empathy_theme_manager_apply_saved (empathy_theme_manager_get (),
-						  view);
-	}
-}
-
 time_t
 empathy_chat_view_get_last_timestamp (EmpathyChatView *view)
 {
@@ -1512,31 +1454,6 @@ empathy_chat_view_set_last_timestamp (EmpathyChatView *view,
 	priv->last_timestamp = timestamp;
 }
 
-EmpathyChatViewBlock
-empathy_chat_view_get_last_block_type (EmpathyChatView *view)
-{
-	EmpathyChatViewPriv *priv;
-
-	g_return_val_if_fail (EMPATHY_IS_CHAT_VIEW (view), 0);
-
-	priv = GET_PRIV (view);
-
-	return priv->last_block_type;
-}
-
-void
-empathy_chat_view_set_last_block_type (EmpathyChatView      *view, 
-				       EmpathyChatViewBlock  block_type)
-{
-	EmpathyChatViewPriv *priv;
-
-	g_return_if_fail (EMPATHY_IS_CHAT_VIEW (view));
-
-	priv = GET_PRIV (view);
-
-	priv->last_block_type = block_type;
-}
-
 EmpathyContact *
 empathy_chat_view_get_last_contact (EmpathyChatView *view)
 {
@@ -1549,22 +1466,3 @@ empathy_chat_view_get_last_contact (EmpathyChatView *view)
 	return priv->last_contact;
 }
 
-void
-empathy_chat_view_set_last_contact (EmpathyChatView *view, EmpathyContact *contact)
-{
-	EmpathyChatViewPriv *priv;
-
-	g_return_if_fail (EMPATHY_IS_CHAT_VIEW (view));
-
-	priv = GET_PRIV (view);
-
-	if (priv->last_contact) {
-		g_object_unref (priv->last_contact);
-		priv->last_contact = NULL;
-	}
-
-	if (contact) {
-		priv->last_contact = g_object_ref (contact);
-	}
-}
-
