@@ -29,7 +29,8 @@
 #include <gtk/gtk.h>
 
 #include <libempathy/empathy-debug.h>
-#include <libempathy-gtk/empathy-accounts-dialog.h>
+
+#include "empathy-accounts-dialog.h"
 
 static void
 destroy_cb (GtkWidget *dialog,
@@ -53,7 +54,7 @@ main (int argc, char *argv[])
 
 	gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (),
 					   PKGDATADIR G_DIR_SEPARATOR_S "icons");
-	dialog = empathy_accounts_dialog_show (NULL);
+	dialog = empathy_accounts_dialog_show (NULL, NULL);
 
 	g_signal_connect (dialog, "destroy",
 			  G_CALLBACK (destroy_cb),
