@@ -26,6 +26,7 @@
 #include <gtk/gtk.h>
 #include <glade/glade.h>
 #include <glib/gi18n.h>
+#include <gdk/gdkkeysyms.h>
 
 #include <telepathy-glib/util.h>
 
@@ -39,11 +40,11 @@
 #include <libempathy-gtk/empathy-presence-chooser.h>
 #include <libempathy-gtk/empathy-conf.h>
 #include <libempathy-gtk/empathy-ui-utils.h>
-#include <libempathy-gtk/empathy-accounts-dialog.h>
 #include <libempathy-gtk/empathy-images.h>
 #include <libempathy-gtk/empathy-new-message-dialog.h>
 #include <libempathy-gtk/empathy-contact-dialogs.h>
 
+#include "empathy-accounts-dialog.h"
 #include "empathy-status-icon.h"
 #include "empathy-preferences.h"
 
@@ -82,71 +83,12 @@ struct _StatusIconEvent {
 	gchar               *message;
 	StatusIconEventFunc  func;
 	gpointer             user_data;
+	TpChannel           *channel;
+	EmpathyStatusIcon   *icon;
 };
 
 G_DEFINE_TYPE (EmpathyStatusIcon, empathy_status_icon, G_TYPE_OBJECT);
 
-static void
-status_icon_event_free (StatusIconEvent *event)
-{
-	g_free (event->icon_name);
-	g_free (event->message);
-	g_slice_free (StatusIconEvent, event);
-}
-
-static void
-status_icon_set_visibility (EmpathyStatusIcon *icon,
-			    gboolean           visible,
-			    gboolean           store)
-{
-	EmpathyStatusIconPriv *priv = GET_PRIV (icon);
-
-	if (store) {
-		empathy_conf_set_bool (empathy_conf_get (),
-				       EMPATHY_PREFS_UI_MAIN_WINDOW_HIDDEN, !visible);
-	}
-
-	if (!visible) {
-		empathy_window_iconify (priv->window, priv->icon);
-	} else {
-		GList *accounts;
-
-		empathy_window_present (GTK_WINDOW (priv->window), TRUE);
-	
-		/* Show the accounts dialog if there is no enabled accounts */
-		accounts = mc_accounts_list_by_enabled (TRUE);
-		if (accounts) {
-			mc_accounts_list_free (accounts);
-		} else {
-			DEBUG ("No enabled account, Showing account dialog");
-			empathy_accounts_dialog_show (GTK_WINDOW (priv->window));
-		}
-	}
-}
-
-static void
-status_icon_notify_visibility_cb (EmpathyConf *conf,
-				  const gchar *key,
-				  gpointer     user_data)
-{
-	EmpathyStatusIcon *icon = user_data;
-	gboolean           hidden = FALSE;
-
-	if (empathy_conf_get_bool (conf, key, &hidden)) {
-		status_icon_set_visibility (icon, !hidden, FALSE);
-	}
-}
-
-static void
-status_icon_toggle_visibility (EmpathyStatusIcon *icon)
-{
-	EmpathyStatusIconPriv *priv = GET_PRIV (icon);
-	gboolean               visible;
-
-	visible = gtk_window_is_active (priv->window);
-	status_icon_set_visibility (icon, !visible, TRUE);
-}
-
 static void
 status_icon_update_tooltip (EmpathyStatusIcon *icon)
 {
@@ -182,6 +124,193 @@ status_icon_update_icon (EmpathyStatusIcon *icon)
 	gtk_status_icon_set_from_icon_name (priv->icon, icon_name);
 }
 
+static gboolean
+status_icon_blink_timeout_cb (EmpathyStatusIcon *icon)
+{
+	EmpathyStatusIconPriv *priv = GET_PRIV (icon);
+
+	priv->showing_event_icon = !priv->showing_event_icon;
+	status_icon_update_icon (icon);
+
+	return TRUE;
+}
+
+static void status_icon_channel_invalidated_cb (TpProxy *channel, guint domain,
+						gint code, gchar *message,
+						EmpathyStatusIcon *icon);
+
+static void
+status_icon_event_free (StatusIconEvent *event)
+{
+	g_free (event->icon_name);
+	g_free (event->message);
+	if (event->channel) {
+		g_signal_handlers_disconnect_by_func (event->channel,
+						      status_icon_channel_invalidated_cb,
+						      event->icon);
+		g_object_unref (event->channel);
+	}
+	g_slice_free (StatusIconEvent, event);
+}
+
+static void
+status_icon_event_remove (EmpathyStatusIcon *icon,
+			  StatusIconEvent   *event)
+{
+	EmpathyStatusIconPriv *priv = GET_PRIV (icon);
+	GSList                *l;
+
+	if (!(l = g_slist_find (priv->events, event))) {
+		return;
+	}
+
+	priv->events = g_slist_delete_link (priv->events, l);		
+	status_icon_event_free (event);
+	status_icon_update_tooltip (icon);
+	status_icon_update_icon (icon);
+
+	if (!priv->events && priv->blink_timeout) {
+		g_source_remove (priv->blink_timeout);
+		priv->blink_timeout = 0;
+	}
+}
+
+static void
+status_icon_event_remove_by_channel (EmpathyStatusIcon *icon,
+				     TpChannel         *channel)
+{
+	EmpathyStatusIconPriv *priv = GET_PRIV (icon);
+	GSList                *l;
+
+	for (l = priv->events; l; l = l->next) {
+		StatusIconEvent *event = l->data;
+
+		if (event->channel &&
+		    empathy_proxy_equal (event->channel, channel)) {
+			DEBUG ("Found event '%s'", event->message);
+			status_icon_event_remove (icon, event);
+			break;
+		}
+	}
+}
+
+static void
+status_icon_event_activate (EmpathyStatusIcon *icon,
+			    StatusIconEvent   *event)
+{
+	if (event->func) {
+		event->func (icon, event->user_data);
+	}
+	status_icon_event_remove (icon, event);
+}
+
+static void
+status_icon_event_add (EmpathyStatusIcon   *icon,
+		       const gchar         *icon_name,
+		       const gchar         *message,
+		       StatusIconEventFunc  func,
+		       gpointer             user_data,
+		       TpChannel           *channel)
+{
+	EmpathyStatusIconPriv *priv = GET_PRIV (icon);
+	StatusIconEvent       *event;
+	gboolean               had_events;
+
+	DEBUG ("Adding event: %s", message);
+
+	event = g_slice_new0 (StatusIconEvent);
+	event->icon_name = g_strdup (icon_name);
+	event->message = g_strdup (message);
+	event->func = func;
+	event->user_data = user_data;
+	event->icon = icon;
+
+	if (channel) {
+		event->channel = g_object_ref (channel);
+		g_signal_connect (channel, "invalidated",
+				  G_CALLBACK (status_icon_channel_invalidated_cb),
+				  icon);
+	}
+
+	had_events = (priv->events != NULL);
+	priv->events = g_slist_append (priv->events, event);
+	if (!had_events) {
+		priv->showing_event_icon = TRUE;
+		status_icon_update_icon (icon);
+		status_icon_update_tooltip (icon);
+
+		if (!priv->blink_timeout) {
+			priv->blink_timeout = g_timeout_add (BLINK_TIMEOUT,
+							     (GSourceFunc) status_icon_blink_timeout_cb,
+							     icon);
+		}
+	}
+}
+
+static void
+status_icon_channel_invalidated_cb (TpProxy           *channel,
+				    guint              domain,
+				    gint               code,
+				    gchar             *message,
+				    EmpathyStatusIcon *icon)
+{
+	DEBUG ("%s", message);
+	status_icon_event_remove_by_channel (icon, TP_CHANNEL (channel));
+}
+
+static void
+status_icon_set_visibility (EmpathyStatusIcon *icon,
+			    gboolean           visible,
+			    gboolean           store)
+{
+	EmpathyStatusIconPriv *priv = GET_PRIV (icon);
+
+	if (store) {
+		empathy_conf_set_bool (empathy_conf_get (),
+				       EMPATHY_PREFS_UI_MAIN_WINDOW_HIDDEN, !visible);
+	}
+
+	if (!visible) {
+		empathy_window_iconify (priv->window, priv->icon);
+	} else {
+		GList *accounts;
+
+		empathy_window_present (GTK_WINDOW (priv->window), TRUE);
+
+		/* Show the accounts dialog if there is no enabled accounts */
+		accounts = mc_accounts_list_by_enabled (TRUE);
+		if (accounts) {
+			mc_accounts_list_free (accounts);
+		} else {
+			DEBUG ("No enabled account, Showing account dialog");
+			empathy_accounts_dialog_show (GTK_WINDOW (priv->window), NULL);
+		}
+	}
+}
+
+static void
+status_icon_notify_visibility_cb (EmpathyConf *conf,
+				  const gchar *key,
+				  gpointer     user_data)
+{
+	EmpathyStatusIcon *icon = user_data;
+	gboolean           hidden = FALSE;
+
+	if (empathy_conf_get_bool (conf, key, &hidden)) {
+		status_icon_set_visibility (icon, !hidden, FALSE);
+	}
+}
+
+static void
+status_icon_toggle_visibility (EmpathyStatusIcon *icon)
+{
+	EmpathyStatusIconPriv *priv = GET_PRIV (icon);
+	gboolean               visible;
+
+	visible = gtk_window_is_active (priv->window);
+	status_icon_set_visibility (icon, !visible, TRUE);
+}
+
 static void
 status_icon_idle_notify_cb (EmpathyStatusIcon *icon)
 {
@@ -198,27 +327,17 @@ status_icon_delete_event_cb (GtkWidget         *widget,
 	return TRUE;
 }
 
-static void
-status_icon_event_activate (EmpathyStatusIcon *icon,
-			    StatusIconEvent   *event)
+static gboolean
+status_icon_key_press_event_cb  (GtkWidget *window,
+				 GdkEventKey *event,
+				 EmpathyStatusIcon *icon)
 {
-	EmpathyStatusIconPriv *priv = GET_PRIV (icon);
-
-	if (event->func) {
-		event->func (icon, event->user_data);
-	}
-
-	priv->events = g_slist_remove (priv->events, event);
-	status_icon_event_free (event);
-	status_icon_update_tooltip (icon);
-	status_icon_update_icon (icon);
-
-	if (!priv->events && priv->blink_timeout) {
-		g_source_remove (priv->blink_timeout);
-		priv->blink_timeout = 0;
+	if (event->keyval == GDK_Escape) {
+		status_icon_set_visibility (icon, FALSE, TRUE);
 	}
+	return FALSE;
 }
-
+				
 static void
 status_icon_activate_cb (GtkStatusIcon     *status_icon,
 			 EmpathyStatusIcon *icon)
@@ -319,51 +438,6 @@ status_icon_create_menu (EmpathyStatusIcon *icon)
 	g_object_unref (glade);
 }
 
-static gboolean
-status_icon_blink_timeout_cb (EmpathyStatusIcon *icon)
-{
-	EmpathyStatusIconPriv *priv = GET_PRIV (icon);
-
-	priv->showing_event_icon = !priv->showing_event_icon;
-	status_icon_update_icon (icon);
-
-	return TRUE;
-}
-
-static void
-status_icon_event_add (EmpathyStatusIcon   *icon,
-		       const gchar         *icon_name,
-		       const gchar         *message,
-		       StatusIconEventFunc  func,
-		       gpointer             user_data)
-{
-	EmpathyStatusIconPriv *priv = GET_PRIV (icon);
-	StatusIconEvent       *event;
-	gboolean               had_events;
-
-	DEBUG ("Adding event: %s", message);
-
-	event = g_slice_new (StatusIconEvent);
-	event->icon_name = g_strdup (icon_name);
-	event->message = g_strdup (message);
-	event->func = func;
-	event->user_data = user_data;
-
-	had_events = (priv->events != NULL);
-	priv->events = g_slist_append (priv->events, event);
-	if (!had_events) {
-		priv->showing_event_icon = TRUE;
-		status_icon_update_icon (icon);
-		status_icon_update_tooltip (icon);
-
-		if (!priv->blink_timeout) {
-			priv->blink_timeout = g_timeout_add (BLINK_TIMEOUT,
-							     (GSourceFunc) status_icon_blink_timeout_cb,
-							     icon);
-		}
-	}
-}
-
 static void
 status_icon_channel_process (EmpathyStatusIcon *icon,
 			     gpointer           user_data)
@@ -372,7 +446,6 @@ status_icon_channel_process (EmpathyStatusIcon *icon,
 	TpChannel             *channel = TP_CHANNEL (user_data);
 
 	empathy_dispatcher_channel_process (priv->dispatcher, channel);
-	g_object_unref (channel);
 }
 
 static gboolean
@@ -404,7 +477,7 @@ status_icon_chat_message_received_cb (EmpathyTpChat     *tp_chat,
 	channel = empathy_tp_chat_get_channel (tp_chat);
 	status_icon_event_add (icon, EMPATHY_IMAGE_NEW_MESSAGE, msg,
 			       status_icon_channel_process,
-			       g_object_ref (channel));
+			       channel, channel);
 
 	g_free (msg);
 }
@@ -442,7 +515,7 @@ status_icon_filter_channel_cb (EmpathyDispatcher *dispatcher,
 
 		status_icon_event_add (icon, EMPATHY_IMAGE_VOIP, msg,
 				       status_icon_channel_process,
-				       g_object_ref (channel));
+				       channel, channel);
 
 		g_free (msg);
 		g_object_unref (contact);
@@ -452,6 +525,14 @@ status_icon_filter_channel_cb (EmpathyDispatcher *dispatcher,
 	g_free (channel_type);
 }
 
+static void
+status_icon_dispatch_channel_cb (EmpathyDispatcher *dispatcher,
+				 TpChannel         *channel,
+				 EmpathyStatusIcon *icon)
+{
+	status_icon_event_remove_by_channel (icon, channel);
+}
+
 static void
 status_icon_tube_process (EmpathyStatusIcon *icon,
 			  gpointer           user_data)
@@ -473,7 +554,8 @@ status_icon_tube_process (EmpathyStatusIcon *icon,
 
 		dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL,
 						 GTK_MESSAGE_ERROR,
-						 GTK_BUTTONS_OK, str);
+						 GTK_BUTTONS_OK,
+						 "%s", str);
 		gtk_window_set_title (GTK_WINDOW (dialog),
 				      _("Invitation Error"));
 		g_free (str);
@@ -512,7 +594,8 @@ status_icon_filter_tube_cb (EmpathyDispatcher     *dispatcher,
 	}
 
 	status_icon_event_add (icon, icon_name, msg, status_icon_tube_process,
-			       empathy_dispatcher_tube_ref (tube));
+			       empathy_dispatcher_tube_ref (tube),
+			       tube->channel);
 
 	g_free (msg);
 }
@@ -558,7 +641,8 @@ status_icon_pendings_changed_cb (EmpathyContactList *list,
 
 	status_icon_event_add (icon, GTK_STOCK_DIALOG_QUESTION, str->str,
 			       status_icon_pending_subscribe,
-			       g_object_ref (contact));
+			       g_object_ref (contact),
+			       NULL);
 
 	g_string_free (str, TRUE);
 }
@@ -650,6 +734,9 @@ empathy_status_icon_init (EmpathyStatusIcon *icon)
 	g_signal_connect (priv->dispatcher, "filter-channel",
 			  G_CALLBACK (status_icon_filter_channel_cb),
 			  icon);
+	g_signal_connect (priv->dispatcher, "dispatch-channel",
+			  G_CALLBACK (status_icon_dispatch_channel_cb),
+			  icon);
 	g_signal_connect (priv->dispatcher, "filter-tube",
 			  G_CALLBACK (status_icon_filter_tube_cb),
 			  icon);
@@ -665,7 +752,7 @@ empathy_status_icon_init (EmpathyStatusIcon *icon)
 }
 
 EmpathyStatusIcon *
-empathy_status_icon_new (GtkWindow *window)
+empathy_status_icon_new (GtkWindow *window, gboolean hide_contact_list)
 {
 	EmpathyStatusIconPriv *priv;
 	EmpathyStatusIcon     *icon;
@@ -678,13 +765,21 @@ empathy_status_icon_new (GtkWindow *window)
 
 	priv->window = g_object_ref (window);
 
+	g_signal_connect (priv->window, "key-press-event",
+			  G_CALLBACK (status_icon_key_press_event_cb),
+			  icon);
+
 	g_signal_connect (priv->window, "delete-event",
 			  G_CALLBACK (status_icon_delete_event_cb),
 			  icon);
 
-	empathy_conf_get_bool (empathy_conf_get (),
-			      EMPATHY_PREFS_UI_MAIN_WINDOW_HIDDEN,
-			      &should_hide);
+	if (!hide_contact_list) {
+		empathy_conf_get_bool (empathy_conf_get (),
+				       EMPATHY_PREFS_UI_MAIN_WINDOW_HIDDEN,
+			               &should_hide);
+	} else {
+		should_hide = TRUE;
+	}
 
 	if (gtk_window_is_active (priv->window) == should_hide) {
 		status_icon_set_visibility (icon, !should_hide, FALSE);
